/**
 * @file sparkhelpermouse.es
 *
 * Contains mouse helper functions.
 *
 * @author Alastair Fettes
 *
 * Copyright (C) 2004  Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * afettes at users.sourceforge.net
 * http://spark.sourceforge.net
 * http://sourceforge.net/projects/spark/
 */

/**
 * Simply contains some mouse helper functions.
 */

/**
 * Convert client X and Y to the co-ordinate space of the
 * passed in DOM node.
 */
function getNodeCoordinates( node, clientX, clientY )
{
    var SVGDoc    = node.ownerDocument;
    var SVGRoot   = SVGDoc.documentElement;

    var targetCTM = getCurrentTransformMatrix(node);
    var inverse   = targetCTM.inverse();

    var p1  = SVGRoot.createSVGPoint();
    p1.x = clientX;
    p1.y = clientY;

    var trans = SVGRoot.currentTranslate;
    p1.x = p1.x - trans.x;
    p1.y = p1.y - trans.y;

    var scale = SVGRoot.currentScale;
    p1.x = p1.x / scale;
    p1.y = p1.y / scale;

    return( p1.matrixTransform(inverse) );
}

/**
 * Get current transform matrix for a node, with workarounds for ASV3
 */
function getCurrentTransformMatrix( node )
{
    var thenode = node;

    // can't call getCTM on, say <tspan>.
    // Need to go up the tree.
    while( thenode != null && thenode.getCTM == null && thenode != svgDocument )
    {
        thenode = thenode.parentNode;
    }

    var ctmMatrix = thenode.getCTM();

    // ASV3.0 workaround.  ASV3 does not properly impliment getCTM().
    // It gives the transform matrix, not the cumulative transform matrix.
    if( navigator.appVersion == '3.0' )
    {
        while( (thenode = thenode.parentNode) != svgDocument )
        {
            ctmMatrix = thenode.getCTM().multiply(ctmMatrix);
        }
    }

    return( ctmMatrix );
}