/**
 * @file button.es
 *
 * Describes the Button Widget type from the SPARK framework.
 *
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.dom.spark.Atom
 * @see net.sourceforge.spark.dom.spark.Observable
 * @see net.sourceforge.spark.dom.spark.Widget
 * @see net.sourceforge.spark.dom.spark.CommandHolder
 *
 * Copyright (C) 2004  Christopher Lewis, Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * http://spark.sourceforge.net
 * afettes at users.sourceforge.net
 */

/**
 * public class Button extends Atom
 *
 * @param in_node The SVG base node for the button widget.
 */
function Button( in_node )
{
	if( in_node != null )
	{
		// inheritance
        this.base = Atom;
        this.base( in_node.getAttribute( "id" ), Button.CLASS );
        
        this.anchor = null;
        this.model  = null;

		for( var i = 0; i < in_node.childNodes.length; i++ )
		{
			var node = in_node.childNodes.item(i);

			if( node.nodeType != 3 )
			{
				if( this.anchor == null && node.nodeName == "desc" )
				{
					this.anchor = node;
				}
				else if( this.model == null && node.nodeName == "metadata" )
				{
					// custom handling for my buttons model information
					this.model = node;
				}
			}
		}

		in_node.addEventListener( "click", this, false );
	}
}

/**
 * public class Button extends SPARK implements Observable
 */
Button.prototype = new Atom;

/**
 * Regular expression defining the class attribute for a Button
 * SVGElement root element.
 */
Button.REGEX = /^SPARK atom Button/;

/**
 * Regular expression defining the class attribute for a Button
 * SVGElement root element.
 */
Button.CLASS = "SPARK atom Button";

/**
 * <p>Handle click events.  </p>
 * <p><strong>This will run commands and notify observers.</strong></p>
 * @param evt The click event.
 */
Button.prototype.click = function( evt )
{
    if( SPARK.requestMouseFocus(this) )
    {
        this.runCommands();
        this.notifyObservers();
        evt.stopPropagation();
        
        SPARK.releaseMouseFocus(this);
    }
};