/**
 * @file atom.es
 *
 * Describes the Atom type from the SPARK framework.
 *
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.dom.spark.Atom
 * @see net.sourceforge.spark.dom.spark.Observable
 * @see net.sourceforge.spark.dom.spark.Observer
 * @see net.sourceforge.spark.dom.spark.Widget
 *
 * Copyright (C) 2004  Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * http://spark.sourceforge.net
 * afettes at users.sourceforge.net
 */

/**
 * An atom may contain no other widgets.
 * @param id The id of the widget.
 * @param type The type of the widget.
 */
function Atom( id, type  )
{
    // inheritance
    if( id && type)
    {
        this.base = Widget;
        this.base( id, type );
    }

    // interface members
    this.m_vpObservers = new Array();
}

// *** ADDED FOR SCRIPTING ***
Atom.prototype = new Widget;
Atom.REGEX     = /^SPARK atom/;

/**
 * The observer to be attached.  It will now be
 * observering this subject.
 * @param in_observer The observer to add.
 */
Atom.prototype.attach = function( in_observer )
{
    var bAdd = true;

    for( var i = 0; i < this.m_vpObservers.length; i++ )
    {
        if( this.m_vpObservers[i] == in_observer )
        {
            bAdd = false;
            break;
        }
    }

    if( bAdd )
    {
        this.m_vpObservers.push( in_observer );
    }
};

/**
 * The observer to be detached.  It will no longer
 * be observering this subject.
 * @param in_observer The observer to remove.
 */
Atom.prototype.detach = function( in_observer )
{
    var bFound = false;

    for( var i = 0; i < this.m_vpObservers.length; i++ )
    {
        if( this.m_vpObservers[i] == in_pObserver && !bFound )
        {
            this.m_vpObservers[i] = null;
            bFound = true;

            if( (i+1) < this.m_vpObservers.length )
            {
                this.m_vpObservers[i]   = this.m_vpObservers[i+1];
                this.m_vpObservers[i+1] = null;
            }
        }
        else if( bFound && (i+1) < this.m_vpObservers.length )
        {
            this.m_vpObservers[i]   = this.m_vpObservers[i+1];
            this.m_vpObservers[i+1] = null;
        }
    }
};

/**
 * Notify all observers of the state change.
 */
Atom.prototype.notifyObservers = function()
{
    for( var i = 0; i < this.m_vpObservers.length; i++ )
    {
		this.m_vpObservers[i].update( this );
    }
};