/**
 * @file widget.es
 *
 * Describes the Widget class from the SPARK framework.
 *
 * @author Christopher Lewis
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.dom.SPARK.Widget
 *
 * Copyright (C) 2004  Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * http://spark.sourceforge.net
 * afettes at users.sourceforge.net
 */

/**
 * public class Widget
 * @param id The id of the widget.
 * @param type The type of the widget.
 */
function Widget( id, type )
{
    this.ID    = id;
    this.TYPE  = type;
    this.state = null;

    // implements CommandHolder
    this.m_vpCommands = new Array();
}

/**
 * Add a command to this widget to be executed.
 * @param in_command The Command to add.
 */
Widget.prototype.addCommand = function( in_command )
{
    var bAdd = true;

    for( var i = 0; i < this.m_vpCommands.length; i++ )
    {
        if( this.m_vpCommands[i] == in_command )
        {
            bAdd = false;
            break;
        }
    }

    if( bAdd )
    {
        this.m_vpCommands.push( in_command );
    }
};

/**
 * Remove a command from this Widget.
 * @param in_command The Command to remove.
 */
Widget.prototype.removeCommand = function( in_command )
{
    var bFound = false;

    for( var i = 0; i < this.m_vpCommands.length; i++ )
    {
        if( !bFound && this.m_vpCommands[i] == in_command )
        {
            this.m_vpCommands[i] = null;
            bFound = true;

            if( (i+1) < this.m_vObservers.length )
            {
                this.m_vpCommands[i]   = this.m_vpCommands[i+1];
                this.m_vpCommands[i+1] = null;
            }
        }
        else if( bFound && (i+1) < this.m_vpCommands.length )
        {
            this.m_vpCommands[i]   = this.m_vpCommands[i+1];
            this.m_vpCommands[i+1] = null;
        }
    }
};

/**
 * Execute the list of commands that this command
 * hold is holding.
 */
Widget.prototype.runCommands = function()
{
    if( this.m_vpCommands.length > 0 )
    {
        for( var i = 0; i < this.m_vpCommands.length; i++ )
        {
            this.m_vpCommands[i].execute();
        }
    }
};

/**
 * Script specific event handler.  This will distribute the
 * given event to the appropriate function.
 */
Widget.prototype.handleEvent = function( evt )
{
    var iType = evt.type;

    if ( this[iType] != null )
    {
        this[iType]( evt );
    }
};

/**
 * Set the state of this widget.
 * @param in_state The new state for the widget.
 */
Widget.prototype.setState = function( in_state )
{
    this.state = in_state;
};

/**
 * Retrieve the state of the widget.
 * @return The current state of the widget.
 */
Widget.prototype.getState = function()
{
    return( this.state );
};